import java.util.Vector;
import java.util.Hashtable;

/** Join operator - join two children; figures out which are join attributes.
 *                  If no join attributes, this is a cartesian product.
 *
 * @author Zack Ives
 * @version 1.0
 */

class Join extends Operator {
    Operator _left;
    Operator _right;
    Table _table;
    Vector _leftJoinAttribs;
    Vector _rightJoinAttribs;
    
    // Constructor - specify left and right children
    //
    // Constructor also maps the variable symbols appropriately for the join output
    public Join(Operator left, Operator right, Hashtable symbolTable) {
        super(symbolTable);
        _table = new Table(left.attribs().name() + "_x_" + right.attribs().name());
        
        // For saving which attributes need to be joined
        _leftJoinAttribs = new Vector();
        _rightJoinAttribs = new Vector();
        
        // Copy the attribute symbols
        _symbols = (Vector)left.variables().clone();
        for (int i = 0; i < left.variables().size(); i++)
            _table.addAttribute(left.attribs().getAttribute(i));
        
        // Figure out which symbols get copied over from the right child
        boolean add;
        for (int i = 0; i < right.variables().size(); i++) {
            add = true;
            // See if we can find a match based on the symbolic variable names
            for (int j = 0; j < left.variables().size(); j++)
                if (left.variables().elementAt(j).equals(right.variables().elementAt(i))) {
                    add = false;
                    
                    // Save that these two join
                    _leftJoinAttribs.addElement(left.attribs().getAttribute(j));
                    _rightJoinAttribs.addElement(right.attribs().getAttribute(i));
                }
            if (add) {
                _symbols.addElement(right.variables().elementAt(i));
                _table.addAttribute(right.attribs().getAttribute(i));
            }
        }
        
        _left = left;
        _right = right;

        // Map symbol table entries to our symbols
        for (int i = 0; i < _symbols.size(); i++) {
            _symbolTable.put(_symbols.elementAt(i), attribs().name() + "." + _table.getAttribute(i));
        }
    }
    
    // Returns the attributes at this node
    public Table attribs() {
        return _table;
    }
    
    // The join attributes from the left child, in order
    public Vector leftJoinAttributes() {
        return _leftJoinAttribs;
    }
    
    // The join attributes from the right child, in order
    public Vector rightJoinAttributes() {
        return _rightJoinAttribs;
    }
    
    // Return the left child operator
    public Operator leftChild() {
        return _left;
    }
    
    // Return the right child operator
    public Operator rightChild() {
        return _right;
    }
    
    // Output string
    public String toString() {
        String temp;
        if (leftJoinAttributes().size() > 0) {
            temp = "Join[";
            for (int i = 0; i < leftJoinAttributes().size(); i++) {
                if (i > 0)
                    temp += ",";
                temp += leftChild().attribs().name() + "." + leftJoinAttributes().elementAt(i) + " = " + 
                    rightChild().attribs().name() + "." + rightJoinAttributes().elementAt(i);
            }
            temp += "]";
        } else
            temp = "Cartesian";
        return temp + "(" + leftChild().toString() + ", " + rightChild().toString() + ")";
    }
}